@@ -21,19 +21,65 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  */
-#ifndef _MSN_H_
-#define _MSN_H_
+#ifndef MSN_H
+#define MSN_H
 
-/* #define MSN_DEBUG_MSG 1 */
-/* #define MSN_DEBUG_SLPMSG 1 */
-/* #define MSN_DEBUG_HTTP 1 */
+typedef enum
+{
+	MSN_LIST_FL_OP = 0x01,
+	MSN_LIST_AL_OP = 0x02,
+	MSN_LIST_BL_OP = 0x04,
+	MSN_LIST_RL_OP = 0x08,
+	MSN_LIST_PL_OP = 0x10
+} MsnListOp;
+#define MSN_LIST_OP_MASK	0x07
 
-/* #define MSN_DEBUG_SLP 1 */
-/* #define MSN_DEBUG_SLP_VERBOSE 1 */
-/* #define MSN_DEBUG_SLP_FILES 1 */
+typedef enum
+{
+	MSN_CLIENT_CAP_WIN_MOBILE = 0x0000001,
+	MSN_CLIENT_CAP_INK_GIF    = 0x0000004,
+	MSN_CLIENT_CAP_INK_ISF    = 0x0000008,
+	MSN_CLIENT_CAP_VIDEO_CHAT = 0x0000010,
+	MSN_CLIENT_CAP_PACKET     = 0x0000020,
+	MSN_CLIENT_CAP_MSNMOBILE  = 0x0000040,
+	MSN_CLIENT_CAP_MSNDIRECT  = 0x0000080,
+	MSN_CLIENT_CAP_WEBMSGR    = 0x0000200,
+	MSN_CLIENT_CAP_TGW        = 0x0000800,
+	MSN_CLIENT_CAP_SPACE      = 0x0001000,
+	MSN_CLIENT_CAP_MCE        = 0x0002000,
+	MSN_CLIENT_CAP_DIRECTIM   = 0x0004000,
+	MSN_CLIENT_CAP_WINKS      = 0x0008000,
+	MSN_CLIENT_CAP_SEARCH     = 0x0010000,
+	MSN_CLIENT_CAP_BOT        = 0x0020000,
+	MSN_CLIENT_CAP_VOICEIM    = 0x0040000,
+	MSN_CLIENT_CAP_SCHANNEL   = 0x0080000,
+	MSN_CLIENT_CAP_SIP_INVITE = 0x0100000,
+	MSN_CLIENT_CAP_SDRIVE     = 0x0400000,
+	MSN_CLIENT_CAP_ONECARE    = 0x1000000,
+	MSN_CLIENT_CAP_P2P_TURN   = 0x2000000,
+	MSN_CLIENT_CAP_P2P_BOOTSTRAP_VIA_UUN = 0x4000000,
+} MsnClientCaps;
 
-/* #define MSN_DEBUG_NS 1 */
-/* #define MSN_DEBUG_SB 1 */
+typedef enum
+{
+	MSN_CLIENT_EXT_CAP_RTC_VIDEO = 0x10,
+	MSN_CLIENT_EXT_CAP_P2PV2     = 0x20
+} MsnClientExtCaps;
+
+typedef enum
+{
+	MSN_CLIENT_VER_5_0  = 0x00,
+	MSN_CLIENT_VER_6_0  = 0x10,	/* MSNC1 */
+	MSN_CLIENT_VER_6_1  = 0x20,	/* MSNC2 */
+	MSN_CLIENT_VER_6_2  = 0x30,	/* MSNC3 */
+	MSN_CLIENT_VER_7_0  = 0x40,	/* MSNC4 */
+	MSN_CLIENT_VER_7_5  = 0x50,	/* MSNC5 */
+	MSN_CLIENT_VER_8_0  = 0x60,	/* MSNC6 */
+	MSN_CLIENT_VER_8_1  = 0x70,	/* MSNC7 */
+	MSN_CLIENT_VER_8_5  = 0x80,	/* MSNC8 */
+	MSN_CLIENT_VER_9_0  = 0x90,	/* MSNC9 */
+	MSN_CLIENT_VER_14_0 = 0xA0	/* MSNC10 */
+} MsnClientVerId;
 
 #include "internal.h"
 
@@ -76,6 +122,8 @@
 
 #define BUDDY_ALIAS_MAXLEN 387
 
+#define MSN_CAM_GUID "4BD96FC0-AB17-4425-A14A-439185962DC8"
+#define MSN_CAM_REQUEST_GUID "1C9AA97E-9C05-4583-A3BD-908A196F1E92"
 #define MSN_FT_GUID "5D3E02AB-6190-11D3-BBBB-00C04F795683"
 #define MSN_OBJ_GUID "A4268EEC-FEC5-49E5-95C3-F126696BDBF6"
 
@@ -86,64 +134,21 @@
 /* Index into attention_types */
 #define MSN_NUDGE 0
 
-typedef enum
-{
-	MSN_LIST_FL_OP = 0x01,
-	MSN_LIST_AL_OP = 0x02,
-	MSN_LIST_BL_OP = 0x04,
-	MSN_LIST_RL_OP = 0x08,
-	MSN_LIST_PL_OP = 0x10
-
-} MsnListOp;
-#define MSN_LIST_OP_MASK	0x07
-
-typedef enum
-{
-	MSN_CLIENT_CAP_WIN_MOBILE = 0x000001,
-	MSN_CLIENT_CAP_INK_GIF    = 0x000004,
-	MSN_CLIENT_CAP_INK_ISF    = 0x000008,
-	MSN_CLIENT_CAP_VIDEO_CHAT = 0x000010,
-	MSN_CLIENT_CAP_PACKET     = 0x000020,
-	MSN_CLIENT_CAP_MSNMOBILE  = 0x000040,
-	MSN_CLIENT_CAP_MSNDIRECT  = 0x000080,
-	MSN_CLIENT_CAP_WEBMSGR    = 0x000200,
-	MSN_CLIENT_CAP_TGW        = 0x000800,
-	MSN_CLIENT_CAP_SPACE      = 0x001000,
-	MSN_CLIENT_CAP_MCE        = 0x002000,
-	MSN_CLIENT_CAP_DIRECTIM   = 0x004000,
-	MSN_CLIENT_CAP_WINKS      = 0x008000,
-	MSN_CLIENT_CAP_SEARCH     = 0x010000,
-	MSN_CLIENT_CAP_BOT        = 0x020000,
-	MSN_CLIENT_CAP_VOICEIM    = 0x040000,
-	MSN_CLIENT_CAP_SCHANNEL   = 0x080000,
-	MSN_CLIENT_CAP_SIP_INVITE = 0x100000,
-	MSN_CLIENT_CAP_SDRIVE     = 0x400000
-
-} MsnClientCaps;
-
-typedef enum
-{
-	MSN_CLIENT_VER_5_0 = 0x00,
-	MSN_CLIENT_VER_6_0 = 0x10,	/* MSNC1 */
-	MSN_CLIENT_VER_6_1 = 0x20,	/* MSNC2 */
-	MSN_CLIENT_VER_6_2 = 0x30,	/* MSNC3 */
-	MSN_CLIENT_VER_7_0 = 0x40,	/* MSNC4 */
-	MSN_CLIENT_VER_7_5 = 0x50,	/* MSNC5 */
-	MSN_CLIENT_VER_8_0 = 0x60,	/* MSNC6 */
-	MSN_CLIENT_VER_8_1 = 0x70,	/* MSNC7 */
-	MSN_CLIENT_VER_8_5 = 0x80	/* MSNC8 */
-
-} MsnClientVerId;
-
 #define MSN_CLIENT_ID_VERSION      MSN_CLIENT_VER_7_0
-#define MSN_CLIENT_ID_CAPABILITIES MSN_CLIENT_CAP_PACKET
+#define MSN_CLIENT_ID_CAPABILITIES (MSN_CLIENT_CAP_PACKET|MSN_CLIENT_CAP_INK_GIF|MSN_CLIENT_CAP_VOICEIM)
 
 #define MSN_CLIENT_ID \
 	((MSN_CLIENT_ID_VERSION    << 24) | \
 	 (MSN_CLIENT_ID_CAPABILITIES))
 
-void msn_act_id(PurpleConnection *gc, const char *entry);
+#define MSN_CLIENT_EXT_ID 0
+
+gboolean msn_email_is_valid(const char *passport);
+void
+msn_set_public_alias(PurpleConnection *gc, const char *alias,
+                     PurpleSetPublicAliasSuccessCallback success_cb,
+                     PurpleSetPublicAliasFailureCallback failure_cb);
 void msn_send_privacy(PurpleConnection *gc);
 void msn_send_im_message(MsnSession *session, MsnMessage *msg);
 
-#endif /* _MSN_H_ */
+#endif /* MSN_H */
