@@ -21,28 +21,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  */
-#ifndef _MSN_SESSION_H_
-#define _MSN_SESSION_H_
+#ifndef MSN_SESSION_H
+#define MSN_SESSION_H
 
 typedef struct _MsnSession MsnSession;
 
-#include "sslconn.h"
-
-#include "user.h"
-#include "slpcall.h"
-
-#include "notification.h"
-#include "switchboard.h"
-#include "group.h"
-
-#include "cmdproc.h"
-#include "nexus.h"
-#include "httpconn.h"
-#include "oim.h"
-
-#include "userlist.h"
-#include "sync.h"
-
 /**
  * Types of errors.
  */
@@ -56,7 +39,6 @@ typedef enum
 	MSN_ERROR_SIGN_OTHER,
 	MSN_ERROR_SERV_DOWN,
 	MSN_ERROR_SERV_UNAVAILABLE
-
 } MsnErrorType;
 
 /**
@@ -69,16 +51,28 @@ typedef enum
 	MSN_LOGIN_STEP_TRANSFER,
 	MSN_LOGIN_STEP_HANDSHAKE2,
 	MSN_LOGIN_STEP_AUTH_START,
-	MSN_LOGIN_STEP_AUTH,
 	MSN_LOGIN_STEP_GET_COOKIE,
 	MSN_LOGIN_STEP_AUTH_END,
 	MSN_LOGIN_STEP_SYN,
 	MSN_LOGIN_STEP_END
-
 } MsnLoginStep;
 
 #define MSN_LOGIN_STEPS MSN_LOGIN_STEP_END
 
+#define MSN_LOGIN_FQY_TIMEOUT 30
+
+#include "group.h"
+#include "httpconn.h"
+#include "nexus.h"
+#include "notification.h"
+#include "oim.h"
+#include "slpcall.h"
+#include "sslconn.h"
+#include "switchboard.h"
+#include "sync.h"
+#include "user.h"
+#include "userlist.h"
+
 struct _MsnSession
 {
 	PurpleAccount *account;
@@ -90,15 +84,17 @@ struct _MsnSession
 
 	gboolean connected;
 	gboolean logged_in; /**< A temporal flag to ignore local buddy list adds. */
+	int      adl_fqy; /**< A count of ADL/FQY so status is only changed once. */
+	guint    login_timeout; /**< Timeout to force status change if ADL/FQY fail. */
 	gboolean destroying; /**< A flag that states if the session is being destroyed. */
 	gboolean http_method;
 
 	MsnNotification *notification;
-	MsnNexus *nexus;
-	MsnOim		*oim;
-	MsnSync *sync;
-
-	MsnUserList *userlist;
+	MsnNexus        *nexus;
+	MsnOim          *oim;
+	MsnSync         *sync;
+	MsnUserList     *userlist;
+	char            *abch_cachekey;
 
 	int servconns_count; /**< The count of server connections. */
 	GList *switches; /**< The list of all the switchboards. */
@@ -107,7 +103,9 @@ struct _MsnSession
 	/*psm info*/
 	char *psm;
 
+#if 0
 	char *blocked_text;
+#endif
 
 	struct
 	{
@@ -123,7 +121,9 @@ struct _MsnSession
 	} passport_info;
 
 	GHashTable *soap_table;
-	int soap_cleanup_handle;
+	guint soap_cleanup_handle;
+
+	GSList *url_datas; /**< PurpleUtilFetchUrlData to be cancelled on exit */
 };
 
 /**
@@ -218,7 +218,16 @@ void msn_session_set_error(MsnSession *session, MsnErrorType error,
 						   const char *info);
 
 /**
- * Sets the current step in the login proccess.
+ * Starts a timeout to initiate finishing login. Sometimes the server ignores
+ * our FQY requests, so this forces ourselves online eventually.
+ *
+ * @param session The MSN session.
+ */
+void
+msn_session_activate_login_timeout(MsnSession *session);
+
+/**
+ * Sets the current step in the login process.
  *
  * @param session The MSN session.
  * @param step The current step.
@@ -236,4 +245,4 @@ void msn_session_finish_login(MsnSession *session);
 void msn_session_report_user(MsnSession *session,const char *passport,
 							const char *msg,PurpleMessageFlags flags);
 
-#endif /* _MSN_SESSION_H_ */
+#endif /* MSN_SESSION_H */
