@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,  USA
  */
 
 #include "internal.h"
@@ -68,7 +68,6 @@ typedef struct _MsnSoapConnection {
 
 	GQueue *queue;
 	MsnSoapRequest *current_request;
-	gboolean unsafe_debug;
 } MsnSoapConnection;
 
 static gboolean msn_soap_connection_run(gpointer data);
@@ -80,7 +79,6 @@ msn_soap_connection_new(MsnSession *session, const char *host)
 	conn->session = session;
 	conn->host = g_strdup(host);
 	conn->queue = g_queue_new();
-	conn->unsafe_debug = g_getenv("PURPLE_MSN_UNSAFE_DEBUG") != NULL;
 	return conn;
 }
 
@@ -385,7 +383,7 @@ msn_soap_process(MsnSoapConnection *conn)
 					msn_soap_connection_handle_next(conn);
 					handled = TRUE;
 					break;
-				} else if (conn->response_code == 503) {
+				} else if (conn->response_code == 503 && conn->session->login_step < MSN_LOGIN_STEP_END) {
 					msn_soap_connection_sanitize(conn, TRUE);
 					msn_session_set_error(conn->session, MSN_ERROR_SERV_UNAVAILABLE, NULL);
 					return;
@@ -509,7 +507,7 @@ msn_soap_read_cb(gpointer data, gint fd, PurpleInputCondition cond)
 		purple_debug_info("soap", "read: %s\n", g_strerror(perrno));
 
 	if (conn->current_request && conn->current_request->secure &&
-		!conn->unsafe_debug)
+		!purple_debug_is_unsafe())
 		purple_debug_misc("soap", "Received secure request.\n");
 	else if (count != 0)
 		purple_debug_misc("soap", "current %s\n", conn->buf->str + cursor);
@@ -659,7 +657,7 @@ msn_soap_connection_run(gpointer data)
 			g_string_append(conn->buf, "\r\n");
 			g_string_append(conn->buf, body);
 
-			if (req->secure && !conn->unsafe_debug)
+			if (req->secure && !purple_debug_is_unsafe())
 				purple_debug_misc("soap", "Sending secure request.\n");
 			else
 				purple_debug_misc("soap", "%s\n", conn->buf->str);
@@ -667,7 +665,8 @@ msn_soap_connection_run(gpointer data)
 			conn->handled_len = 0;
 			conn->current_request = req;
 
-			purple_input_remove(conn->event_handle);
+			if (conn->event_handle)
+				purple_input_remove(conn->event_handle);
 			conn->event_handle = purple_input_add(conn->ssl->fd,
 				PURPLE_INPUT_WRITE, msn_soap_write_cb, conn);
 			if (!msn_soap_write_cb_internal(conn, conn->ssl->fd, PURPLE_INPUT_WRITE, TRUE)) {
