@@ -278,7 +278,7 @@ silcpurple_chat_chpk_cb(SilcPurpleChauth sgc, PurpleRequestFields *fields)
 	SilcClient client = sg->client;
 	SilcClientConnection conn = sg->conn;
 	PurpleRequestField *f;
-	const GList *list;
+	GList *list;
 	SilcPublicKey public_key;
 	SilcBuffer chpks, pk, chidp;
 	SilcUInt16 c = 0, ct;
@@ -449,7 +449,7 @@ void silcpurple_chat_chauth_show(SilcPurple sg, SilcChannelEntry channel,
 		g_snprintf(tmp2, sizeof(tmp2), "%s\n  %s\n  %s",
 			   ident->realname ? ident->realname : ident->username ?
 			   ident->username : "", fingerprint, babbleprint);
-		purple_request_field_list_add(f, tmp2, pubkey);
+		purple_request_field_list_add_icon(f, tmp2, NULL, pubkey);
 
 		silc_free(fingerprint);
 		silc_free(babbleprint);
@@ -651,7 +651,7 @@ silcpurple_chat_permanent(PurpleBlistNode *node, gpointer data)
 
 typedef struct {
 	SilcPurple sg;
-	const char *channel;
+	char *channel;
 } *SilcPurpleChatInput;
 
 static void
@@ -700,7 +700,7 @@ silcpurple_chat_ulimit(PurpleBlistNode *node, gpointer data)
 
 	SilcPurpleChatInput s;
 	SilcChannelEntry channel;
-	const char *ch;
+	char *ch;
 	char tmp[32];
 
 	g_return_if_fail(PURPLE_BLIST_NODE_IS_CHAT(node));
@@ -1351,7 +1351,7 @@ int silcpurple_chat_send(PurpleConnection *gc, int id, const char *msg, PurpleMe
 					       flags, (unsigned char *)msg2,
 					       strlen(msg2), TRUE);
 	if (ret) {
-		serv_got_chat_in(gc, id, purple_connection_get_display_name(gc), 0, msg,
+		serv_got_chat_in(gc, id, purple_connection_get_display_name(gc), msgflags, msg,
 				 time(NULL));
 	}
 	g_free(tmp);
