@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02111-1301, USA.
  */
 #include "internal.h"
 #include "gtkblist.h"
@@ -68,7 +68,7 @@ new_person_cb(GtkWidget *w, GevoAddBuddyDialog *dialog)
 	const char *group_name;
 
 	group_name =
-		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dialog->group_combo)->entry));
+		pidgin_text_combo_box_entry_get_text(dialog->group_combo);
 
 	gevo_new_person_dialog_show(dialog->book, NULL, dialog->account, dialog->username,
 								(*group_name ? group_name : NULL),
@@ -105,7 +105,7 @@ select_buddy_cb(GtkWidget *w, GevoAddBuddyDialog *dialog)
 					   -1);
 
 	group_name =
-		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dialog->group_combo)->entry));
+		pidgin_text_combo_box_entry_get_text(dialog->group_combo);
 
 	if (username == NULL || *username == '\0')
 	{
@@ -234,6 +234,7 @@ populate_treeview(GevoAddBuddyDialog *dialog, const gchar *uri)
 	EBook *book;
 	gboolean status;
 	GList *cards, *c;
+	GError *err = NULL;
 
 	if (dialog->book != NULL)
 	{
@@ -250,10 +251,11 @@ populate_treeview(GevoAddBuddyDialog *dialog, const gchar *uri)
 
 	gtk_list_store_clear(dialog->model);
 
-	if (!gevo_load_addressbook(uri, &book, NULL))
+	if (!gevo_load_addressbook(uri, &book, &err))
 	{
 		purple_debug_error("evolution",
-						 "Error retrieving default addressbook\n");
+						 "Error retrieving default addressbook: %s\n", err->message);
+		g_error_free(err);
 
 		return;
 	}
@@ -312,11 +314,11 @@ populate_treeview(GevoAddBuddyDialog *dialog, const gchar *uri)
 		}
 		else
 		{
-			add_ims(dialog, contact, name, aims,    "prpl-oscar");
+			add_ims(dialog, contact, name, aims,    "prpl-aim");
 			add_ims(dialog, contact, name, jabbers, "prpl-jabber");
 			add_ims(dialog, contact, name, yahoos,  "prpl-yahoo");
 			add_ims(dialog, contact, name, msns,    "prpl-msn");
-			add_ims(dialog, contact, name, icqs,    "prpl-oscar");
+			add_ims(dialog, contact, name, icqs,    "prpl-icq");
 			add_ims(dialog, contact, name, novells, "prpl-novell");
 		}
 	}
@@ -394,11 +396,11 @@ search_changed_cb(GtkEntry *entry, GevoAddBuddyDialog *dialog)
 		}
 		else
 		{
-			add_ims(dialog, contact, name, aims,    "prpl-oscar");
+			add_ims(dialog, contact, name, aims,    "prpl-aim");
 			add_ims(dialog, contact, name, jabbers, "prpl-jabber");
 			add_ims(dialog, contact, name, yahoos,  "prpl-yahoo");
 			add_ims(dialog, contact, name, msns,    "prpl-msn");
-			add_ims(dialog, contact, name, icqs,    "prpl-oscar");
+			add_ims(dialog, contact, name, icqs,    "prpl-icq");
 			add_ims(dialog, contact, name, novells, "prpl-novell");
 		}
 	}
@@ -442,10 +444,7 @@ gevo_add_buddy_dialog_show(PurpleAccount *account, const char *username,
 	if (username != NULL)
 		dialog->username = g_strdup(username);
 
-	dialog->win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_role(GTK_WINDOW(dialog->win), "add_buddy");
-	gtk_window_set_title(GTK_WINDOW(dialog->win), _("Add Buddy"));
-	gtk_container_set_border_width(GTK_CONTAINER(dialog->win), 12);
+	dialog->win = pidgin_create_window(_("Add Buddy"), PIDGIN_HIG_BORDER, "add_buddy", TRUE);
 	gtk_widget_set_size_request(dialog->win, -1, 400);
 
 	g_signal_connect(G_OBJECT(dialog->win), "delete_event",
@@ -547,20 +546,11 @@ gevo_add_buddy_dialog_show(PurpleAccount *account, const char *username,
 	gtk_combo_box_set_active(GTK_COMBO_BOX(dialog->addrbooks_combo), 0);
 
 	/* Group box */
-	hbox = gtk_hbox_new(FALSE, 6);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-	gtk_widget_show(hbox);
-
-	label = gtk_label_new(_("Group:"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
-
-	dialog->group_combo = gtk_combo_new();
-	gtk_combo_set_popdown_strings(GTK_COMBO(dialog->group_combo),
-								  gevo_get_groups());
-	gtk_box_pack_start(GTK_BOX(hbox), dialog->group_combo, TRUE, TRUE, 0);
-	gtk_widget_show(dialog->group_combo);
+	dialog->group_combo =
+		pidgin_text_combo_box_entry_new(group, gevo_get_groups());
+	pidgin_add_widget_to_vbox(GTK_BOX(vbox), _("Group:"), NULL,
+							  dialog->group_combo, TRUE, NULL);
+	gtk_widget_show_all(dialog->group_combo);
 
 	/* Cool. Now we only have a little left... */
 
